@@ -3,14 +3,15 @@ These are the basic black box tests for the doNd functions.
 """
 import hypothesis.strategies as hst
 import matplotlib.pyplot as plt
+import matplotlib
 import numpy as np
 import pytest
-from hypothesis import given, settings
+from hypothesis import HealthCheck, given, settings
 
 from qcodes import config
 from qcodes.dataset.data_set import DataSet
+from qcodes.dataset import new_experiment
 from qcodes.instrument.parameter import Parameter
-from qcodes.tests.dataset.conftest import empty_temp_db, experiment
 from qcodes.tests.instrument_mocks import (ArraySetPointParam,
                                            Multi2DSetPointParam,
                                            Multi2DSetPointParam2Sizes,
@@ -21,9 +22,6 @@ from qcodes.utils.validators import Arrays
 
 from .conftest import ArrayshapedParam
 
-temp_db = empty_temp_db
-temp_exp = experiment
-
 
 @pytest.fixture(autouse=True)
 def set_tmp_output_dir(tmpdir):
@@ -94,35 +92,45 @@ def test_param_callable(_param_callable):
     assert _param_modified.get() == 2
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
-@pytest.mark.parametrize('period, plot', [(None, True), (None, False),
-                         (1, True), (1, False)])
-def test_do0d_with_real_parameter(_param, period, plot):
-    do0d(_param, write_period=period, do_plot=plot)
+@pytest.mark.usefixtures("plot_close", "experiment")
+@pytest.mark.parametrize('period', [None, 1])
+@pytest.mark.parametrize('plot', [None, True, False])
+@pytest.mark.parametrize('plot_config', [None, True, False])
+def test_do0d_with_real_parameter(_param, period, plot, plot_config):
+
+    if plot_config is not None:
+        config.dataset.dond_plot = plot_config
+
+    output = do0d(_param, write_period=period, do_plot=plot)
+    assert len(output[1]) == 1
+    if plot is True or plot is None and plot_config is True:
+        assert isinstance(output[1][0], matplotlib.axes.Axes)
+    else:
+        assert output[1][0] is None
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+@pytest.mark.usefixtures("plot_close", "experiment")
 @pytest.mark.parametrize('period, plot', [(None, True), (None, False),
                          (1, True), (1, False)])
 def test_do0d_with_complex_parameter(_param_complex, period, plot):
     do0d(_param_complex, write_period=period, do_plot=plot)
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+@pytest.mark.usefixtures("plot_close", "experiment")
 @pytest.mark.parametrize('period, plot', [(None, True), (None, False),
                          (1, True), (1, False)])
 def test_do0d_with_a_callable(_param_callable, period, plot):
     do0d(_param_callable, write_period=period, do_plot=plot)
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+@pytest.mark.usefixtures("plot_close", "experiment")
 @pytest.mark.parametrize('period, plot', [(None, True), (None, False),
                          (1, True), (1, False)])
 def test_do0d_with_2_parameters(_param, _param_complex, period, plot):
     do0d(_param, _param_complex, write_period=period, do_plot=plot)
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+@pytest.mark.usefixtures("plot_close", "experiment")
 @pytest.mark.parametrize('period, plot', [(None, True), (None, False),
                          (1, True), (1, False)])
 def test_do0d_with_parameter_and_a_callable(_param_complex, _param_callable,
@@ -130,25 +138,25 @@ def test_do0d_with_parameter_and_a_callable(_param_complex, _param_callable,
     do0d(_param_callable, _param_complex, write_period=period, do_plot=plot)
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+@pytest.mark.usefixtures("plot_close", "experiment")
 def test_do0d_output_type_real_parameter(_param):
     data = do0d(_param)
     assert isinstance(data[0], DataSet) is True
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+@pytest.mark.usefixtures("plot_close", "experiment")
 def test_do0d_output_type_complex_parameter(_param_complex):
     data_complex = do0d(_param_complex)
     assert isinstance(data_complex[0], DataSet) is True
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+@pytest.mark.usefixtures("plot_close", "experiment")
 def test_do0d_output_type_callable(_param_callable):
     data_func = do0d(_param_callable)
     assert isinstance(data_func[0], DataSet) is True
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+@pytest.mark.usefixtures("plot_close", "experiment")
 def test_do0d_output_data(_param):
     exp = do0d(_param)
     data = exp[0]
@@ -157,12 +165,12 @@ def test_do0d_output_data(_param):
     assert loaded_data == np.array([_param.get()])
 
 
-@pytest.mark.usefixtures("temp_exp", "temp_db")
+@pytest.mark.usefixtures("experiment")
 @pytest.mark.parametrize("multiparamtype", [MultiSetPointParam,
                                             Multi2DSetPointParam,
                                             Multi2DSetPointParam2Sizes])
 @given(n_points_pws=hst.integers(min_value=1, max_value=1000))
-@settings(deadline=None)
+@settings(deadline=None, suppress_health_check=(HealthCheck.function_scoped_fixture,))
 def test_do0d_verify_shape(_param, _param_complex, multiparamtype,
                            dummyinstrument, n_points_pws):
     arrayparam = ArraySetPointParam(name='arrayparam')
@@ -179,8 +187,8 @@ def test_do0d_verify_shape(_param, _param_complex, multiparamtype,
     for i, name in enumerate(multiparam.full_names):
         expected_shapes[name] = tuple(multiparam.shapes[i])
     expected_shapes['arrayparam'] = tuple(arrayparam.shape)
-    expected_shapes['simple_parameter'] = ()
-    expected_shapes['simple_complex_parameter'] = ()
+    expected_shapes['simple_parameter'] = (1,)
+    expected_shapes['simple_complex_parameter'] = (1,)
     expected_shapes[paramwsetpoints.full_name] = (n_points_pws, )
     assert results[0].description.shapes == expected_shapes
 
@@ -195,7 +203,7 @@ def test_do0d_verify_shape(_param, _param_complex, multiparamtype,
             assert param_data.shape == expected_shapes[name]
 
 
-@pytest.mark.usefixtures("temp_exp", "temp_db")
+@pytest.mark.usefixtures("experiment")
 def test_do0d_parameter_with_array_vals():
     param = ArrayshapedParam(name='paramwitharrayval', vals=Arrays(shape=(10,)))
     results = do0d(param)
@@ -203,7 +211,24 @@ def test_do0d_parameter_with_array_vals():
     assert results[0].description.shapes == expected_shapes
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+def test_do0d_explicit_experiment(_param, experiment):
+    experiment_2 = new_experiment('new-exp', 'no-sample')
+
+    data1 = do0d(_param, do_plot=False, exp=experiment)
+    assert data1[0].exp_name == "test-experiment"
+    data2 = do0d(_param, do_plot=False, exp=experiment_2)
+    assert data2[0].exp_name == "new-exp"
+    # by default the last experiment is used
+    data3 = do0d(_param, do_plot=False)
+    assert data3[0].exp_name == "new-exp"
+
+
+def test_do0d_explicit_name(_param, experiment):
+    data1 = do0d(_param, do_plot=False, measurement_name="my measurement")
+    assert data1[0].name == "my measurement"
+
+
+@pytest.mark.usefixtures("plot_close", "experiment")
 @pytest.mark.parametrize('delay', [0, 0.1, 1])
 def test_do1d_with_real_parameter(_param_set, _param, delay):
 
@@ -214,7 +239,27 @@ def test_do1d_with_real_parameter(_param_set, _param, delay):
     do1d(_param_set, start, stop, num_points, delay, _param)
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+@pytest.mark.usefixtures("plot_close", "experiment")
+@pytest.mark.parametrize('plot', [None, True, False])
+@pytest.mark.parametrize('plot_config', [None, True, False])
+def test_do1d_plot(_param_set, _param, plot, plot_config):
+
+    if plot_config is not None:
+        config.dataset.dond_plot = plot_config
+
+    start = 0
+    stop = 1
+    num_points = 1
+
+    output = do1d(_param_set, start, stop, num_points, 0, _param, do_plot=plot)
+    assert len(output[1]) == 1
+    if plot is True or plot is None and plot_config is True:
+        assert isinstance(output[1][0], matplotlib.axes.Axes)
+    else:
+        assert output[1][0] is None
+
+
+@pytest.mark.usefixtures("plot_close", "experiment")
 @pytest.mark.parametrize('delay', [0, 0.1, 1])
 def test_do1d_with_complex_parameter(_param_set, _param_complex, delay):
 
@@ -225,7 +270,7 @@ def test_do1d_with_complex_parameter(_param_set, _param_complex, delay):
     do1d(_param_set, start, stop, num_points, delay, _param_complex)
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+@pytest.mark.usefixtures("plot_close", "experiment")
 @pytest.mark.parametrize('delay', [0, 0.1, 1])
 def test_do1d_with_2_parameter(_param_set, _param, _param_complex, delay):
 
@@ -236,7 +281,7 @@ def test_do1d_with_2_parameter(_param_set, _param, _param_complex, delay):
     do1d(_param_set, start, stop, num_points, delay, _param, _param_complex)
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+@pytest.mark.usefixtures("plot_close", "experiment")
 @pytest.mark.parametrize('delay', [0, 0.1, 1])
 def test_do1d_output_type_real_parameter(_param_set, _param, delay):
 
@@ -248,7 +293,7 @@ def test_do1d_output_type_real_parameter(_param_set, _param, delay):
     assert isinstance(data[0], DataSet) is True
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+@pytest.mark.usefixtures("plot_close", "experiment")
 def test_do1d_output_data(_param, _param_set):
 
     start = 0
@@ -266,13 +311,27 @@ def test_do1d_output_data(_param, _param_set):
     np.testing.assert_array_equal(loaded_data[_param_set.name], np.linspace(0, 1, 5))
 
 
-@pytest.mark.usefixtures("temp_exp", "temp_db")
+def test_do0d_parameter_with_setpoints_2d(dummyinstrument):
+    dummyinstrument.A.dummy_start(0)
+    dummyinstrument.A.dummy_stop(10)
+    dummyinstrument.A.dummy_n_points(10)
+    dummyinstrument.A.dummy_start_2(2)
+    dummyinstrument.A.dummy_stop_2(7)
+    dummyinstrument.A.dummy_n_points_2(3)
+    dataset, _, _ = do0d(dummyinstrument.A.dummy_parameter_with_setpoints_2d)
+
+    data = dataset.cache.data()['dummyinstrument_ChanA_dummy_parameter_with_setpoints_2d']
+    for array in data.values():
+        assert array.shape == (10, 3)
+
+
+@pytest.mark.usefixtures("experiment")
 @pytest.mark.parametrize("multiparamtype", [MultiSetPointParam,
                                             Multi2DSetPointParam,
                                             Multi2DSetPointParam2Sizes])
 @given(num_points=hst.integers(min_value=1, max_value=10),
        n_points_pws=hst.integers(min_value=1, max_value=1000))
-@settings(deadline=None)
+@settings(deadline=None, suppress_health_check=(HealthCheck.function_scoped_fixture,))
 def test_do1d_verify_shape(_param, _param_complex, _param_set, multiparamtype,
                            dummyinstrument, num_points, n_points_pws):
     arrayparam = ArraySetPointParam(name='arrayparam')
@@ -300,7 +359,7 @@ def test_do1d_verify_shape(_param, _param_complex, _param_set, multiparamtype,
     assert results[0].description.shapes == expected_shapes
 
 
-@pytest.mark.usefixtures("temp_exp", "temp_db")
+@pytest.mark.usefixtures("experiment")
 def test_do1d_parameter_with_array_vals(_param_set):
     param = ArrayshapedParam(name='paramwitharrayval', vals=Arrays(shape=(10,)))
     start = 0
@@ -323,7 +382,38 @@ def test_do1d_parameter_with_array_vals(_param_set):
             assert param_data.shape == expected_shapes[name]
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+def test_do1d_explicit_experiment(_param_set, _param, experiment):
+    start = 0
+    stop = 1
+    num_points = 5
+    delay = 0
+
+    experiment_2 = new_experiment('new-exp', 'no-sample')
+
+    data1 = do1d(_param_set, start, stop, num_points, delay,
+                 _param, do_plot=False, exp=experiment)
+    assert data1[0].exp_name == "test-experiment"
+    data2 = do1d(_param_set, start, stop, num_points, delay,
+                 _param, do_plot=False, exp=experiment_2)
+    assert data2[0].exp_name == "new-exp"
+    # by default the last experiment is used
+    data3 = do1d(_param_set, start, stop, num_points, delay,
+                 _param, do_plot=False)
+    assert data3[0].exp_name == "new-exp"
+
+
+def test_do1d_explicit_name(_param_set, _param, experiment):
+    start = 0
+    stop = 1
+    num_points = 5
+    delay = 0
+
+    data1 = do1d(_param_set, start, stop, num_points, delay,
+                 _param, do_plot=False, measurement_name="my measurement")
+    assert data1[0].name == "my measurement"
+
+
+@pytest.mark.usefixtures("plot_close", "experiment")
 @pytest.mark.parametrize('sweep, columns', [(False, False), (False, True),
                          (True, False), (True, True)])
 def test_do2d(_param, _param_complex, _param_set, _param_set_2, sweep, columns):
@@ -343,7 +433,38 @@ def test_do2d(_param, _param_complex, _param_set, _param_set_2, sweep, columns):
          _param, _param_complex, set_before_sweep=sweep, flush_columns=columns)
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+@pytest.mark.usefixtures("plot_close", "experiment")
+@pytest.mark.parametrize('plot', [None, True, False])
+@pytest.mark.parametrize('plot_config', [None, True, False])
+def test_do2d_plot(_param_set, _param_set_2, _param, plot, plot_config):
+
+    if plot_config is not None:
+        config.dataset.dond_plot = plot_config
+
+    start_p1 = 0
+    stop_p1 = 1
+    num_points_p1 = 1
+    delay_p1 = 0
+
+    start_p2 = 0.1
+    stop_p2 = 1.1
+    num_points_p2 = 2
+    delay_p2 = 0
+
+    output = do2d(
+        _param_set, start_p1, stop_p1, num_points_p1, delay_p1,
+        _param_set_2, start_p2, stop_p2, num_points_p2, delay_p2,
+        _param, do_plot=plot
+    )
+
+    assert len(output[1]) == 1
+    if plot is True or plot is None and plot_config is True:
+        assert isinstance(output[1][0], matplotlib.axes.Axes)
+    else:
+        assert output[1][0] is None
+
+
+@pytest.mark.usefixtures("plot_close", "experiment")
 def test_do2d_output_type(_param, _param_complex, _param_set, _param_set_2):
 
     start_p1 = 0
@@ -362,7 +483,7 @@ def test_do2d_output_type(_param, _param_complex, _param_set, _param_set_2):
     assert isinstance(data[0], DataSet) is True
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+@pytest.mark.usefixtures("plot_close", "experiment")
 def test_do2d_output_data(_param, _param_complex, _param_set, _param_set_2):
 
     start_p1 = 0
@@ -410,7 +531,7 @@ def test_do2d_output_data(_param, _param_complex, _param_set, _param_set_2):
     )
 
 
-@pytest.mark.usefixtures("temp_exp", "temp_db")
+@pytest.mark.usefixtures("experiment")
 @pytest.mark.parametrize('sweep, columns', [(False, False), (False, True),
                          (True, False), (True, True)])
 @pytest.mark.parametrize("multiparamtype", [MultiSetPointParam,
@@ -419,7 +540,7 @@ def test_do2d_output_data(_param, _param_complex, _param_set, _param_set_2):
 @given(num_points_p1=hst.integers(min_value=1, max_value=10),
        num_points_p2=hst.integers(min_value=1, max_value=10),
        n_points_pws=hst.integers(min_value=1, max_value=1000))
-@settings(deadline=None)
+@settings(deadline=None, suppress_health_check=(HealthCheck.function_scoped_fixture,))
 def test_do2d_verify_shape(_param, _param_complex, _param_set, _param_set_2,
                            multiparamtype,
                            dummyinstrument,
@@ -468,7 +589,7 @@ def test_do2d_verify_shape(_param, _param_complex, _param_set, _param_set_2,
             assert param_data.shape == expected_shapes[name]
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+@pytest.mark.usefixtures("plot_close", "experiment")
 def test_do1d_additional_setpoints(_param, _param_complex, _param_set):
     additional_setpoints = [Parameter(
         f'additional_setter_parameter_{i}',
@@ -495,8 +616,9 @@ def test_do1d_additional_setpoints(_param, _param_complex, _param_set):
             plt.close('all')
 
 
+@settings(suppress_health_check=(HealthCheck.function_scoped_fixture,))
 @given(num_points_p1=hst.integers(min_value=1, max_value=10))
-@pytest.mark.usefixtures("temp_exp", "temp_db")
+@pytest.mark.usefixtures("experiment")
 def test_do1d_additional_setpoints_shape(_param, _param_complex, _param_set,
                                          num_points_p1):
     arrayparam = ArraySetPointParam(name='arrayparam')
@@ -526,7 +648,7 @@ def test_do1d_additional_setpoints_shape(_param, _param_complex, _param_set,
     assert results[0].description.shapes == expected_shapes
 
 
-@pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
+@pytest.mark.usefixtures("plot_close", "experiment")
 def test_do2d_additional_setpoints(_param, _param_complex,
                                    _param_set, _param_set_2):
     additional_setpoints = [Parameter(
@@ -558,8 +680,8 @@ def test_do2d_additional_setpoints(_param, _param_complex,
 
 @given(num_points_p1=hst.integers(min_value=1, max_value=10),
        num_points_p2=hst.integers(min_value=1, max_value=10))
-@settings(deadline=None)
-@pytest.mark.usefixtures("temp_exp", "temp_db")
+@settings(deadline=None, suppress_health_check=(HealthCheck.function_scoped_fixture,))
+@pytest.mark.usefixtures("experiment")
 def test_do2d_additional_setpoints_shape(
         _param, _param_complex,
         _param_set, _param_set_2,
@@ -595,3 +717,48 @@ def test_do2d_additional_setpoints_shape(
         'simple_parameter': (1, 1, num_points_p1, num_points_p2)
     }
     assert results[0].description.shapes == expected_shapes
+
+
+def test_do2d_explicit_experiment(_param_set, _param_set_2, _param, experiment):
+    start_p1 = 0
+    stop_p1 = 0.5
+    num_points_p1 = 5
+    delay_p1 = 0
+
+    start_p2 = 0.5
+    stop_p2 = 1
+    num_points_p2 = 5
+    delay_p2 = 0.0
+
+    experiment_2 = new_experiment('new-exp', 'no-sample')
+
+    data1 = do2d(_param_set, start_p1, stop_p1, num_points_p1, delay_p1,
+                 _param_set_2, start_p2, stop_p2, num_points_p2, delay_p2,
+                 _param, do_plot=False, exp=experiment)
+    assert data1[0].exp_name == "test-experiment"
+    data2 = do2d(_param_set, start_p1, stop_p1, num_points_p1, delay_p1,
+                 _param_set_2, start_p2, stop_p2, num_points_p2, delay_p2,
+                 _param, do_plot=False, exp=experiment_2)
+    assert data2[0].exp_name == "new-exp"
+    # by default the last experiment is used
+    data3 = do2d(_param_set, start_p1, stop_p1, num_points_p1, delay_p1,
+                 _param_set_2, start_p2, stop_p2, num_points_p2, delay_p2,
+                 _param, do_plot=False)
+    assert data3[0].exp_name == "new-exp"
+
+
+def test_do2d_explicit_name(_param_set, _param_set_2, _param, experiment):
+    start_p1 = 0
+    stop_p1 = 0.5
+    num_points_p1 = 5
+    delay_p1 = 0
+
+    start_p2 = 0.5
+    stop_p2 = 1
+    num_points_p2 = 5
+    delay_p2 = 0.0
+
+    data1 = do2d(_param_set, start_p1, stop_p1, num_points_p1, delay_p1,
+                 _param_set_2, start_p2, stop_p2, num_points_p2, delay_p2,
+                 _param, do_plot=False, measurement_name="my measurement")
+    assert data1[0].name == "my measurement"
